function accuracies = run_experiment(stdev, num_segments, segment_length, edge_prob_same, edge_prob_diff, experiment_id, outf)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Generates a model and calls various inference algorithms.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%edge_prob_same = 0.2;
%edge_prob_diff = 0.1;
edge_mean_same = 1;
edge_mean_diff = 1;
edge_weight_stdev = 0;

edge_weights = [1e-10 1e-9 1e-8 1e-7 1e-6 1e-5 1e-4 1e-3 1e-2 1e-1 1.0];
%edge_weights = [1e-3];
%mp_weights = [0.000001 0.0001 0.01 0.1 1];
mp_weights = [1e-2, 1e-4, 1e-6];
graph_scales = [1e-2, 1e-4, 1e-6];

transition_prob = 1/segment_length;
%N = num_segments*2*segment_length;
N = num_segments;

true_labels = sample_simple_model(N,transition_prob);
true_labels = true_labels';
%true_labels = zeros(1,N);
%for i = 1:num_segments
%    true_labels((i-1)*segment_length*2+1:i*segment_length*2) = ...
%         [repmat(1,1,segment_length) repmat(2,1,segment_length)];
% end

% TODO: It would be better to have multiple labels and 
% multiple tracks. 


observed_vals = true_labels + normrnd(0,stdev,1,N);
lab1_pots = normpdf(observed_vals,1,stdev);
lab2_pots = normpdf(observed_vals,2,stdev);
partitions = lab1_pots + lab2_pots;
pots = [(lab1_pots./partitions)' (lab2_pots./partitions)'];

% Generate random weight matrix
fprintf('Generating weight matrix...\n', stdev)
if (false)
    W = sparse(rand(N,N) <= edge_prob_same);
    W(logical(sparse(1:N,1:N,1))) = 0;
    W = sparse((W+W') >= 1);
else
    W = zeros(N,N);
    for i=1:N;
    for j=(i+1):N;
        if true_labels(i) == true_labels(j)
            if (rand() <= edge_prob_same)
                val = normrnd(edge_mean_same, edge_weight_stdev);
                if (val > 0)
                    W(i,j) = val;
                    W(j,i) = val;
                end
            end
        else
            if (rand() <= edge_prob_diff)
                val = normrnd(edge_mean_diff, edge_weight_stdev);
                if (val > 0)
                    W(i,j) = val;
                    W(j,i) = val;
                end
            end
            
        end
    end
    end
end

% Functions for evaluating results
lik_of = @(bel) prod(bel(:,1)'-(true_labels-1));
accuracy_of = @(bel) sum(~xor((bel(:,1)'>= 0.5), ~(true_labels-1)));
% print_result = @(name, bel) fprintf('%s:\t%f\t%f\n', name, accuracy_of(bel), log(lik_of(bel)));
print_result = @(name, bel) '';

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Run different inference algorithms and return accuracies
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

write_accuracy = @(method, hyperparameter1, hyperparameter2, accuracy) fprintf(outf, '%d\t%f\t%d\t%d\t%f\t%f\t%s\t%e\t%e\t%d\n', ...
            experiment_id, ...
            stdev, ...
            num_segments, ...
            segment_length, ...
            edge_prob_same, ...
            edge_prob_diff, ...
            method, ...
            hyperparameter1, ...
            hyperparameter2, ...
            accuracy);


fprintf('Running simple models...\n', stdev)
trivial_bel = pots;
print_result('trivial', trivial_bel);
write_accuracy('trivial', 0, 0, accuracy_of(trivial_bel));

chain_bel = infer_simple_model(pots, N, transition_prob);
print_result('chain', chain_bel);
write_accuracy('chain', 0, 0, accuracy_of(chain_bel));

fprintf('Running LBP...\n', stdev)
for edge_weight = edge_weights;
    lbp_bel = infer_lbp_model(pots, W, edge_weight, transition_prob, N);
    print_result(sprintf('lbp-%04g',edge_weight), lbp_bel);
    write_accuracy('lbp', edge_weight, 0, accuracy_of(lbp_bel));
end

% for edge_weight = edge_weights;
%     mf_bel = infer_mf_model(pots, W, edge_weight, transition_prob, N);
%     print_result(sprintf('mf-%04g',edge_weight), mf_bel);
%     accuracies = [accuracies accuracy_of(mf_bel)];
% end


fprintf('Running GBR...\n', stdev)
for do_sql=[true false];
    if do_sql;
        method = 'egbr_sql';
    else
        method = 'egbr';
    end
    for mp_weight = mp_weights;
        for graph_scale = graph_scales;
            egbr_bel = infer_egbr(pots, W, mp_weight, graph_scale, transition_prob, N, do_sql);
            print_result(sprintf('egbr-%04g',mp_weight), egbr_bel);
            write_accuracy(method, mp_weight, graph_scale, accuracy_of(egbr_bel));
        end
    end
end




